#!/usr/bin/python
# -*- coding: utf-8 -*-
###################################
#
#  DRIZA
#
###################################


#Copyright (C) 2006  Inmaculada Luengo Merino, Nstor Arocha Rodrguez
#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION="0.3"
CODENAME="tambucho de popa"


#Imports
#Librerias del sistema
import sys

#Librerias del programa
import datos
import config
import interfaz
#Directorio de ventanas y dilogos
sys.path.append("./dialog/vprincipal")
from vprincipal import VPrincipal
from vsalida import VSalida
from dsplash import DSplash
from qt import *

class Driza(QApplication):
	"""Clase - aplicacin principal. 
	El widget principal es Vprincipal, que es el dialogo principal
	"""
	def __init__(self,config,options):
		""" Constructor de la clase general
		Recibe la configuracion, e inicializa las variables
		options son las opciones escogidas en la linea de comandos
		"""
		QApplication.__init__(self,[]) 
		#La configuracion. Se espera un ConfigHandler
		self.config=config
		#Datos es el objeto que guarda los datos
		self.dato=datos.contenedorregvar()
		#Se espera el objeto variable que actua como filtro
		self.filtro=None
		#Variable que controla el fichero que esta abierto
		self.fichero = False  # Variable de control sobre si hay un fichero abierto. No sube a driza porque puede ser que otros dialogos abran otros ficheros
		#Inicializacion de interfaces. Actuan como elementos conversores
		self.ifi=interfaz.IFile(self.config,self.dato)
		self.isa=interfaz.IOutput(self.config,self.dato)
		# Inicializacion de ventanas
		self.maindialog = VPrincipal(self)
		self.vsalida = VSalida()
		self.splash = DSplash(self.maindialog,"Dialogo Splash",0,)
		self.setMainWidget(self.maindialog)
		self.copiadato=False
		self.copiavars=False
		self.undo=False #Guarda si estamos en undo o no

	def bucle(self):
		""" 
		Procedimiento Bucle principal. 
		Lanzamos la ventana splash despues de la principal para que obtenga el foco
		"""
		self.maindialog.show()
		if options.vsplash==None: 
			if (self.config["vsplash"] and not self.fichero):
				self.splash.show()
		elif options.vsplash:
			self.splash.show()
		if self.config.new:
			#Es la primera vez que se ejecuta el programa
			rc = QMessageBox.information(self.maindialog,'Bienvenido', 'Parece que es la primera vez que ejecutas driza')
		self.exec_loop() 

	def cargarfichero(self,fichero=None):
		"""
		Realiza la acción de cargar el fichero self.fichero.
		Si se indica otro fichero, se actualizará self.fichero
		"""
		if fichero: self.fichero=fichero
		self.dato=self.ifi.imp(self.fichero) #
		self.config["lfichero"].insert(self.fichero)
		self.maindialog.show()
	
	def guardarfichero(self,fichero=None):
		"""
		Realiza la accion de guardar los datos en el fichero self.fichero
		Si se indica otro fichero, se actualizará self.fichero
		"""
		if fichero: self.fichero=fichero
		import os
		if os.path.exists(self.fichero): #El fichero existe
			rc = QMessageBox.warning(self.maindialog,'Atencion', 'Sobreescribir?', 'Si','Cancelar','',0,1 )
			if rc==0:
				self.ifi.exp(self.fichero)
		else:
			self.ifi.exp(self.fichero)


	def calcular(self,accion,variables,opciones):
		"""Esta funcion recibe del dialogo de calculo los datos necesarios y una vez calculados los importa a la salida
		-accion es una cadena con el nombre de la accion
		-variables es una lista con el nombre de las variables
		-opciones es un diccionario con las opciones
		"""
		objeto=self.isa.exp(accion,variables,opciones)
		self.vsalida.app_obj(objeto) #Aadir a la salida el resultado
		self.vsalida.show()

	def crearvariable(self,nombre=None,tipo="Entero",valorpordefecto=0,descripcion=None,expresion=None):
		"""Crea la variable nombre a partir de la expresion cadena
		Si no se pasa cadena, se crea una variable con los valores por defecto
		"""
		self.dato.app_var(nombre,tipo,{},valorpordefecto,descripcion,expresion)
	
	def guardarestado(self):
		import copy
		self.copiadato=copy.copy(self.dato.datos)
		self.copiavars=copy.copy(self.dato.vars)
		self.undo=False

	def restaurarestado(self):
		temp=self.dato.datos
		self.dato.datos=self.copiadato
		self.copiadato=temp
		temp=self.dato.vars
		self.dato.vars=self.copiavars
		self.copiavars=temp

		self.undo=True

	def version(self):
		return VERSION
	
if __name__ == "__main__":
	#Obtencion de parametros de ejecucion
	from optparse import OptionParser
	usage = "usage: %prog [opciones] [fichero]"
	version="Driza "+VERSION+" Codename:"+CODENAME+"\n Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodrguez"
	parser = OptionParser(usage=usage,version=version)
	parser.add_option("-s", "--splash", action="store_true", dest="vsplash", help="Mostrar la ventana de splash")
	parser.add_option("-n", "--nosplash", action="store_false", dest="vsplash", help="No mostrar la ventana de splash")
	(options,args) = parser.parse_args()

	#Creacion del objetos. 
	conf=config.ConfigHandler()

	#Creacion de la interfaz de usuario y Llamada al loop de QT
	ventanaprincipal=Driza(conf,options)
	if args: 
		ventanaprincipal.cargarfichero(args[0])
	ventanaprincipal.bucle()
