#!/usr/bin/python
# -*- coding: latin-1 -*-
#
#Fichero de interfaces
#Copyright (C) 2006  Inmaculada Luengo Merino, Néstor Arocha Rodríguez

#This file is part of Driza.
#
#Driza is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#Driza is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with Driza; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from qt import *
from qttable import *
import sys
import os
import salida
from rpy import *  


#Clase base de interfaz
class Interfaz:
	"""Clase genérica de la que heredan los interfaces """
	def __init__(self,config,datos):
		self.config=config
		self.datos=datos

class IOutput(Interfaz):
	"""Interfaz con la Salida.R es solo un intermediario"""

	def __init__(self,config,datos):
		Interfaz.__init__(self,config,datos)

	def exp(self,accion,variables,opciones):
		"""Exportacion de interfaz de datos/salida"""
		# El estudio descriptivo Devuelve los datos en una tabla
		# Aqui se controla la accion que va a realizar
		if accion=="descriptivo":
			resultado=salida.descriptivo(opciones["media"],opciones["varianza"],opciones["desviacion"],opciones["mediana"])
			sys.path.append("./lib")
			media=None
			varianza=None
			desviacion=None
			mediana=None
			lvariables = [ x.latin1() for x in variables ] # Sin Qstring
			i=0
			for variable in lvariables:
				lista=[int(x) for x in self.datos.get_col(lvariables[i])]
				if opciones["media"]:
					media=r.mean(lista)
				if opciones["varianza"]:
					varianza=r.var(lista)
				if opciones["desviacion"]:
					desviacion=r.sd(lista)
				if opciones["mediana"]:
					mediana=r.median(lista)
				resultado.insertarregistro(lvariables[i],media,varianza,desviacion,mediana)
				i+=1
			return resultado 
		elif accion=="jicuadrado":
			resultado=salida.jicuadrado()
			lvariables = [ x.latin1() for x in variables ] # Sin Qstring
			num=self.datos.num_dat()
			i=0
			for variable in lvariables:
				lista=[int(x) for x in self.datos.get_col(lvariables[i])]
				if opciones["distribucion"]=="uniforme":
					distribucion=r.runif(num,1./num,1./num)
					resultadoprueba=r.chisq_test(lista,p=distribucion)
					resultado.insertarregistro(lvariables[i],opciones["percentil"],resultadoprueba)
				i+=1
			return resultado
		else:
			print "NO IMPLEMENTADO"
			return False

class IFile(Interfaz):
	""" Interfaz entre datos y ficheros """

	def __init__(self,config,datos):
		import re
		Interfaz.__init__(self,config,datos)

		#Objetos expresiones regulares, para determinar el tipo de fichero
		self.regexp=re.compile('.*\.pkl$')

	def imp(self,fichero):
		"""Coge los datos de un fichero y los guarda en el objeto dato"""
		#Obtener la extension
		#obtener el tipo a partir de una funcion de la libreria de formatos #PENDIENTE
		archivo = open(fichero, 'r')
		if self.regexp.match(fichero):
			import pickle
			self.datos=pickle.load(archivo) #Devuelve copia... corregir
		archivo.close
		return self.datos #SUPERCHAPUZA. deberia trabajar con el objeto....  pero me devuelve unaa copia?¿
			
			
	def exp(self,fichero):
		"""Guarda el contenido de un objeto dato en un fichero"""

		archivo = open(fichero, 'w')
		if self.regexp.match(fichero):
			import pickle
			pickle.dump(self.datos,archivo)
		archivo.close()
		
